try:
    import importlib
except ImportError:
    from django.utils import importlib  # noqa


def import_attribute(path):
    pkg, attr = path.rsplit(".", 1)
    ret = getattr(importlib.import_module(pkg), attr)
    return ret



from django.contrib import admin

from .models import Activity, Notice


class NoticeAdmin(admin.ModelAdmin):
    date_hierarchy = "created_at"
    list_display = ("activity", "user", "created_at", "read_at", "archived_at")
    raw_id_fields = ("user", "activity")
    list_filter = ("created_at",)


class ActivityAdmin(admin.ModelAdmin):
    date_hierarchy = "created_at"
    list_display = ("type", "created_at")
    list_filter = ("created_at",)


admin.site.register(Notice, NoticeAdmin)
admin.site.register(Activity, ActivityAdmin)



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('actistream', '0002_notice_index_together'),
    ]

    operations = [
        migrations.RenameIndex(
            model_name='notice',
            new_name='notice',
            old_fields=('user', 'read_at'),
        ),
    ]



from django.test import TestCase
from django.contrib.auth import get_user_model
from django.core import mail

from actistream.models import Notice, Activity

from . import activities
from .models import Comment, Article


User = get_user_model()


class ActivityTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create(email="john@doe.com", username="john")
        self.article = Article.objects.create()
        self.comment = Comment.objects.create(article=self.article, user=self.user)
        self.activity = activities.CommentPosted.create(
            target=self.article, actor=self.user, action_object=self.comment
        )

    def test_notice(self):
        notice_recipients = [self.user]
        Notice.objects.send(self.activity, notice_recipients)
        msg = mail.outbox[0]
        self.assertEqual(msg.subject, "Comment posted")
        assert msg.to == ["john@doe.com"]
        assert msg.content_subtype == "html"

    def test_for_target(self):
        qs = Activity.objects.for_target(self.article)
        assert qs.count() == 1
        assert qs.filter(pk=self.activity.pk).exists()

    def test_for_action_object(self):
        qs = Activity.objects.for_action_object(self.comment)
        assert qs.count() == 1
        assert qs.filter(pk=self.activity.pk).exists()

    def test_for_action_objects(self):
        qs = Activity.objects.for_action_objects(Comment.objects.all())
        assert qs.count() == 1
        assert qs.filter(pk=self.activity.pk).exists()

    def test_fetch_related(self):
        # TODO: Actually test something
        Activity.objects.fetch_related(Activity.objects.all())



import json

from actistream import registry

from .adapter import get_adapter


class ActivityTypeMeta(type):
    def __new__(mcs, name, bases, d):
        userdef = bases and bases[0] != object
        if userdef:
            d["app"] = d["__module__"].split(".")[-2]
            d["code"] = name.lower()
            d["id"] = d["app"] + "." + d["code"]
        ret = type.__new__(mcs, name, bases, d)
        if userdef:
            registry.register(ret)
        return ret


class ActivityWrapper(object):
    def __init__(self, activity):
        self.activity = activity

    def __getattr__(self, name):
        aliases = self.activity.get_type().aliases
        if name in aliases:
            return getattr(self.activity, aliases[name])
        return self.__dict__[name]

    def get_context_data(self):
        return {"activity": self.activity}

    def is_active(self):
        return True

    def get_user_email(self, user):
        return user.email

    def get_notice_mail_context_data(self, notice, request=None):
        return {
            "activity": self.activity,
            "activity_context": self.get_context_data(),
            "notice": notice,
            "user": notice.user,
        }

    def render_notice_mail(self, notice, request=None):
        ctx = self.get_notice_mail_context_data(notice, request)
        activity_type = self.activity.get_type()
        return get_adapter(request).render_mail(
            "{0}/activities/{1}".format(activity_type.app, activity_type.code),
            self.get_user_email(notice.user),
            context=ctx,
        )

    def send_notice_mail(self, notice, request=None):
        msg = self.render_notice_mail(notice, request)
        msg.send()
        return msg


class ActivityType(metaclass=ActivityTypeMeta):
    aliases = {}
    Wrapper = ActivityWrapper

    @classmethod
    def create(cls, **kwargs):
        from .models import Activity

        commit = kwargs.pop("commit", True)
        activity_kwargs = {}
        for k, v in kwargs.items():
            activity_kwargs[cls.aliases.get(k, k)] = v
        extra_data = activity_kwargs.get("extra_data")
        if extra_data is not None and not isinstance(extra_data, str):
            activity_kwargs["extra_data"] = json.dumps(extra_data)
        activity = Activity(type=cls.id, **activity_kwargs)
        if commit:
            activity.save()
        return activity



# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-09-15 20:40
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Activity",
            options={"verbose_name": "activity", "verbose_name_plural": "activities"},
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("actor_id", models.PositiveIntegerField()),
                ("target_id", models.PositiveIntegerField()),
                ("action_object_id", models.PositiveIntegerField()),
                ("type", models.CharField(max_length=100, verbose_name="type")),
                ("flags", models.BigIntegerField(default=0)),
                (
                    "extra_data",
                    models.TextField(blank=True, verbose_name="additional data"),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="created at"),
                ),
                (
                    "action_object_ct",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="contenttypes.ContentType",
                    ),
                ),
                (
                    "actor_ct",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="contenttypes.ContentType",
                    ),
                ),
                (
                    "target_ct",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Notice",
            options={"verbose_name": "notice", "verbose_name_plural": "notices"},
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(verbose_name="created at")),
                (
                    "read_at",
                    models.DateTimeField(blank=True, null=True, verbose_name="read at"),
                ),
                (
                    "archived_at",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="archived at"
                    ),
                ),
                (
                    "activity",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="actistream.Activity",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
        ),
    ]



from importlib import import_module

VERSION = (3, 0, 1, "final", 0)

__title__ = "django-actistream"
__version_info__ = VERSION
__version__ = ".".join(map(str, VERSION[:3])) + (
    "-{}{}".format(VERSION[3], VERSION[4] or "") if VERSION[3] != "final" else ""
)
__author__ = "Raymond Penners"
__license__ = "MIT"
__copyright__ = "Copyright 2017-2023 Raymond Penners and contributors"


class ActivityTypeRegistry(object):
    def __init__(self):
        self.type_map = {}

    def get_list(self):
        return self.type_map.values()

    def register(self, cls):
        self.type_map[cls.id] = cls()

    def by_id(self, id):
        return self.type_map[id]

    def as_choices(self):
        for nt in self.get_list():
            yield (nt.id, nt.verbose_name)

    def load(self):
        from django.conf import settings

        for app in settings.INSTALLED_APPS:
            try:
                m = app + ".activities"
                import_module(m)
            except ImportError:
                pass


registry = ActivityTypeRegistry()
